function z = poly_root_new(P)

% poly_root - find the roots of a polynomial
%
%   z = poly_root(C);
%
%   computes the roots of the polynomial whose coefficients
%   are the elements of the vector C. If C has N+1 components,
%   the polynomial is C(1)*X^N + ... + C(N)*X + C(N+1).
%
%   Same input structure as roots().
%
%   Special code for degrees <= 4.
%   This method can take a list of polynomials, 
%   i.e. C(:,i) is the coefficient of the ith polynomial
%   (fast parallel resolution).
%
%
%   Copyright (c) 2011 Gabriel Peyre
%
%   EX : aa=randn(1,4);roots(aa),poly_root_new(aa)
%

if(size(P,1)==1 && size(P,2)~=1)
  P             = P';
end

if(size(P,1)==2)     % degree 1
  P             = P(end:-1:1,:);    
  z             = -P(1,:)./P(2,:);
elseif(size(P,1)==3) % degree 2
  P             = P(end:-1:1,:);
  delta         = P(2,:).^2 - 4*P(1,:).*P(3,:);
  sqdtt         = sqrt(delta);
  z             = [(-P(2,:)+sqdtt)./(2*P(3,:)); (-P(2,:)-sqdtt)./(2*P(3,:)) ];
elseif(size(P,1)==4) % degree 3     
  j             = exp(2*i*pi/3);
  z             = zeros(3,size(P,2));
  % normalize polynomial
  for k= 2:4
    P(k,:)      = P(k,:)./P(1,:);
  end
  P(1,:)        = 1;
  P32           =  P(2,:).*P(2,:);
  p             = -P(2,:).*P(2,:)/3+P(3,:);
  q             = 2*P(2,:).^3/27-P(2,:).*P(3,:)/3+P(4,:);
  delt          = q.^2 + 4/27*p.^3;

  Ip            = find(delt>0);
  if(isempty(Ip)==0)
    u           = nthroot((-q(Ip) + sqrt(delt(Ip)))/2,3);
    v           = nthroot((-q(Ip) - sqrt(delt(Ip)))/2,3);
    z(1,Ip)     = u + v               - P(2,Ip)/3;
    z(2,Ip)     = j*u   + conj(j)*v   - P(2,Ip)/3;
    z(3,Ip)     = j^2*u + conj(j^2)*v - P(2,Ip)/3;
  end
  Im            = find(delt<0);
  if(isempty(Im)==0)
    u           = ((-q(Im) + i*sqrt(-delt(Im)))/2).^(1/3);
    z(1,Im)     = u   + conj(u)       - P(2,Im)/3;
    z(2,Im)     = j*u + conj(j*u)     - P(2,Im)/3;
    z(3,Im)     = j^2*u + conj(j^2*u) - P(2,Im)/3;
  end
  Iz            = find(delt==0);
  if(isempty(Iz)==0)
    u           = ((-q(Iz) + i*sqrt(-delt(Iz)))/2).^(1/3);
    z(1,Iz)     =  3*q(Iz)./p(Iz)      - P(2,Iz)/3;
    z(2,Iz)     = -3*q(Iz)./(2*p(Iz))  - P(2,Iz)/3;
    z(3,Iz)     = -3*q(Iz)./(2*p(Iz))  - P(2,Iz)/3;
  end
elseif(size(P,1)==5) % degree 4
  a             = P(1,:);
  b             = P(2,:);
  c             = P(3,:);
  d             = P(4,:);
  e             = P(5,:);
  
  
  ff            = (1./2).*sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)));
  gg            = (1./2).*b.^2./a.^2-(4./3).*c./a-(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a-(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3));
  hh            = (c.*b./a.^2-2.*d./a-(1./4).*b.^3./a.^3)./sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))); 
  z(1,:)        = -(1./4).*b./a + ff + 0.5*sqrt(gg + hh);
  z(2,:)        = -(1./4).*b./a + ff - 0.5*sqrt(gg + hh);
  z(3,:)	= -(1./4).*b./a - ff + 0.5*sqrt(gg - hh);
  z(4,:)	= -(1./4).*b./a - ff - 0.5*sqrt(gg - hh);
  
  %z(1,:)        = -(1./4).*b./a+(1./2).*sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))+(1./2).*sqrt((1./2).*b.^2./a.^2-(4./3).*c./a-(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a-(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))+(c.*b./a.^2-2.*d./a-(1./4).*b.^3./a.^3)./sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))); 
  %z(2,:)        = -(1./4).*b./a+(1./2).*sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))-(1./2).*sqrt((1./2).*b.^2./a.^2-(4./3).*c./a-(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a-(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))+(c.*b./a.^2-2.*d./a-(1./4).*b.^3./a.^3)./sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))); 
  %z(3,:)        = -(1./4).*b./a-(1./2).*sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))+(1./2).*sqrt((1./2).*b.^2./a.^2-(4./3).*c./a-(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a-(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))-(c.*b./a.^2-2.*d./a-(1./4).*b.^3./a.^3)./sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))); 
  %z(4,:)        = -(1./4).*b./a-(1./2).*sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)))-(1./2).*sqrt((1./2).*b.^2./a.^2-(4./3).*c./a-(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a-(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))-(c.*b./a.^2-2.*d./a-(1./4).*b.^3./a.^3)./sqrt((1./4).*b.^2./a.^2-(2./3).*c./a+(1./6).*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3)./a+(2./3).*(-3.*d.*b+12.*a.*e+c.^2)./(a.*(-36.*d.*c.*b-288.*e.*c.*a+108.*d.^2.*a+108.*e.*b.^2+8.*c.^3+12.*sqrt(-3.*d.^2.*b.^2.*c.^2+12.*e.*b.^2.*c.^3-54.*d.*c.*b.^3.*e+12.*d.^3.*b.^3+81.*e.^2.*b.^4+12.*d.^2.*a.*c.^3+384.*a.^2.*e.^2.*c.^2-48.*a.*e.*c.^4+240.*d.*b.*a.*e.*c.^2+18.*d.^2.*b.^2.*a.*e+576.*d.*b.*e.^2.*a.^2-54.*d.^3.*c.*b.*a-432.*e.*c.*a.^2.*d.^2-432.*e.^2.*c.*a.*b.^2-768.*a.^3.*e.^3+81.*d.^4.*a.^2)).^(1./3))));
  
  % round very small imaginary entries
  I             = find(abs(imag(z))<1e-8);
  z(I)          = real(z(I));
else
  warning('Works only for degree <=4, using roots');
  z             = zeros(size(P,1)-1, size(P,2));
  for k = 1:size(P,2)
    z(:,k)      = roots(P(:,k));
  end
end

% call roots if problems
It              = find(sum(or(isnan(z),isinf(z)),1)>0);
for k = 1:numel(It)
  z(:,It(k))    = roots(P(:,It(k)));
end


%----------------------------------------------
function z3 = recubsqrt(z)


if(sum(abs(imag(z)))>0)
  error('Pb in recubsqrt.m : you ask for a complex cubic root not a real one...');
end
j               = exp(2*i*pi/3);
z3              = z.^(1/3);
for k = 1:2
  I1            = find(abs(imag(z3))>1e-12);
  z3(I1)        = j*z3(I1);
end
z3              = real(z3);